@@ -1,7 +1,7 @@
 import io
 import uuid
 import boto3
-from typing import List, Optional
+from typing import List
 
 import dbt
 import agate
@@ -16,7 +16,6 @@ from dbt.adapters.glue.gluedbapi import GlueConnection
 from dbt.adapters.glue.relation import SparkRelation
 from dbt.exceptions import NotImplementedException
 from dbt.adapters.base.impl import catch_as_completed
-from botocore.exceptions import ClientError
 from dbt.utils import executor
 from dbt.events import AdapterLogger
 
@@ -46,41 +45,6 @@ class GlueAdapter(SQLAdapter):
     def date_function(cls) -> str:
         return 'current_timestamp()'
 
-    def get_connection(self):
-        connection: GlueConnectionManager = self.connections.get_thread_connection()
-        session: GlueConnection = connection.handle
-        client = boto3.client("glue", region_name=session.credentials.region)
-        cursor = session.cursor()
-
-        return session, client, cursor
-
-    def list_schemas(self, database: str) -> List[str]:
-        # results[1] contains the result of the query
-        # row[0] get the first column of the result
-        # here schemas contains all the databases in glue
-        results = self.connections.execute("show databases", fetch=True)
-        schemas = [row[0] for row in results[1]]
-        return schemas
-
-    def list_relations_without_caching(self, schema_relation: BaseRelation):
-        session, client, cursor = self.get_connection()
-        relations = []
-        try:
-            response = client.get_tables(
-                DatabaseName=schema_relation.schema,
-            )
-            for table in response.get("TableList", []):
-                relations.append(self.Relation.create(
-                    schema=schema_relation.schema,
-                    identifier=table.get("Name"),
-                    type=self.relation_type_map.get(table.get("TableType")),
-                ))
-        except Exception as e:
-            logger.error(e)
-            logger.error("list_relations_without_caching exception")
-
-        return relations
-
     @classmethod
     def convert_text_type(cls, agate_table, col_idx):
         return "string"
@@ -102,19 +66,54 @@ class GlueAdapter(SQLAdapter):
     def convert_datetime_type(cls, agate_table, col_idx):
         return "timestamp"
 
-    def check_schema_exists(self, database: str, schema: str) -> bool:
+    def get_connection(self):
+        connection: GlueConnectionManager = self.connections.get_thread_connection()
+        session: GlueConnection = connection.handle
+        client = boto3.client("glue", region_name=session.credentials.region)
+        cursor = session.cursor()
 
+        return session, client, cursor
+
+    def list_schemas(self, database: str) -> List[str]:
+        # results[1] contains the result of the query
+        # row[0] get the first column of the result
+        # here schemas contains all the databases in glue
+        results = self.connections.execute("show databases", fetch=True)
+        schemas = [row[0] for row in results[1]]
+        return schemas
+
+    def list_relations_without_caching(self, schema_relation: SparkRelation):
+        logger.debug("++++++ list_relations_without_caching")
+        session, client, cursor = self.get_connection()
+        relations = []
         try:
-            list = list_schemas(schema)
+            response = client.get_tables(
+                DatabaseName=schema_relation.schema,
+            )
+            for table in response.get("TableList", []):
+                relations.append(self.Relation.create(
+                    database=schema_relation.schema,
+                    schema=schema_relation.schema,
+                    identifier=table.get("Name"),
+                    type=self.relation_type_map.get(table.get("TableType")),
+                ))
+            return relations
+        except Exception as e:
+            logger.error(e)
+
+    def check_schema_exists(self, database: str, schema: str) -> bool:
+        logger.debug("++++++ check_schema_exists")
+        try:
+            list = self.list_schemas(schema)
             if 'schema' in list:
                 return True
             else:
                 return False
         except Exception as e:
             logger.error(e)
-            logger.error("check_schema_exists exception")
 
     def check_relation_exists(self, relation: BaseRelation) -> bool:
+        logger.debug("++++++ check_relation_exists")
         try:
             relation = self.get_relation(
                 database=relation.schema,
@@ -127,103 +126,15 @@ class GlueAdapter(SQLAdapter):
                 return True
         except Exception as e:
             logger.error(e)
-            logger.error("check_relation_exists exception")
-
-
-    def get_relation(self, database, schema, identifier):
-        relations = []
-        session, client, cursor = self.get_connection()
-        try:
-            response = client.get_table(
-                DatabaseName=schema,
-                Name=identifier
-            )
-            logger.debug(f"""debug type
-                             {response.get('Table', {}).get('TableType', 'Table')}
-            """)
-            relations.append(self.Relation.create(
-                schema=schema,
-                identifier=identifier,
-                type=self.relation_type_map.get(response.get("Table", {}).get("TableType", "Table"))
-            ))
-            logger.debug(f"""schema : {schema}
-                             identifier : {identifier}
-                             type : {self.relation_type_map.get(response.get('Table', {}).get('TableType', 'Table'))}
-            """)
-            return relations
-        except Exception as e:
-            logger.error(e)
-            logger.error(f"relation {schema}.{identifier} not found")
 
     @available
-    def drop_view(self, relation: BaseRelation):
-        session, client, cursor = self.get_connection()
-        code = f'''DROP VIEW IF EXISTS {relation.schema}.{relation.name}'''
-        try:
-            cursor.execute(code)
-        except Exception as e:
-            logger.error(e)
-            logger.error("drop_view exception")
-            logger.error(f"""drop_view exception
-                             relation schema : {relation.schema}
-                             relation identfier : {relation.name}
-            """)
-
-    @available
-    def drop_relation(self, relations):
-        session, client, cursor = self.get_connection()
-        for relation in relations:
-            code = f'''DROP TABLE IF EXISTS {relation}'''
-            try:
-                cursor.execute(code)
-            except Exception as e:
-                logger.error(e)
-                logger.error(f"""drop_relation exception
-                                 relation schema : {relation.schema}
-                                 relation identfier : {relation.name}
-                """)
-
-    @available
-    def create_view_as(self, relation: BaseRelation, sql: str):
-        session, client, cursor = self.get_connection()
-        if self.check_relation_exists(relation):
-            code = f'''DROP VIEW IF EXISTS {relation.schema}.{relation.name}'''
-            try:
-                cursor.execute(code)
-            except Exception as e:
-                logger.error(e)
-                logger.error(f"""drop table exception
-                                 relation schema : {relation.schema}
-                                 relation identfier : {relation.name}
-                """)
-
-        code = f'''
-        create or replace view {relation.schema}.{relation.name}
-        as
-        {sql}
-        '''
-
-        try:
-            cursor.execute(code)
-        except Exception as e:
-            logger.error(e)
-            logger.error(f"""create_view exception
-                             relation schema : {relation.schema}
-                             relation identfier : {relation.name}
-            """)
-
-        check_code = f'''select * from {relation.schema}.{relation.name} limit 1'''
-
-        return check_code
-
-    def rename_relation(self, from_relation, to_relation):
+    def glue_rename_relation(self, from_relation, to_relation):
         logger.debug("rename " + from_relation.schema + " to " + to_relation.identifier)
         session, client, cursor = self.get_connection()
         code = f'''
         custom_glue_code_for_dbt_adapter
         df = spark.sql("""select * from {from_relation.schema}.{from_relation.name}""")
         df.registerTempTable("df")
-        df = df.coalesce(1)
         table_name = '{to_relation.schema}.{to_relation.name}'
         writer = (
                         df.write.mode("append")
@@ -231,6 +142,8 @@ class GlueAdapter(SQLAdapter):
                         .option("path", "{session.credentials.location}/{to_relation.schema}/{to_relation.name}/")
                     )
         writer.saveAsTable(table_name, mode="append")
+        spark.sql("""drop table {from_relation.schema}.{from_relation.name}""")
+        SqlWrapper2.execute("""select * from {to_relation.schema}.{to_relation.name} limit 1""")
         '''
         try:
             cursor.execute(code)
@@ -238,9 +151,35 @@ class GlueAdapter(SQLAdapter):
             logger.error(e)
             logger.error("rename_relation exception")
 
-    def get_columns_in_relation(self, relation: BaseRelation):
-        logger.debug(f"Command launched: describe {relation.schema}.{relation.identifier}")
+    def get_relation(self, database, schema, identifier):
+        logger.debug("++++++ get_relation")
+        relations = []
         session, client, cursor = self.get_connection()
+        try:
+            response = client.get_table(
+                DatabaseName=schema,
+                Name=identifier
+            )
+            relations.append(self.Relation.create(
+                database=schema,
+                schema=schema,
+                identifier=identifier,
+                type=self.relation_type_map.get(response.get("Table", {}).get("TableType", "Table"))
+            ))
+            logger.debug(f"""schema : {schema}
+                             identifier : {identifier}
+                             type : {self.relation_type_map.get(response.get('Table', {}).get('TableType', 'Table'))}
+                        """)
+            return relations
+        except client.exceptions.EntityNotFoundException as e:
+            logger.debug(e)
+        except Exception as e:
+            logger.error(e)
+
+    def get_columns_in_relation(self, relation: BaseRelation) -> [Column]:
+        logger.debug("++++++ get_columns_in_relation")
+        session, client, cursor = self.get_connection()
+        # https://spark.apache.org/docs/3.0.0/sql-ref-syntax-aux-describe-table.html
         code = f'''describe {relation.schema}.{relation.identifier}'''
         columns = []
         try:
@@ -252,7 +191,6 @@ class GlueAdapter(SQLAdapter):
                         columns.append(column)
         except Exception as e:
             logger.error(e)
-            logger.error("get_columns_in_relation exception")
 
         # strip hudi metadata columns.
         columns = [x for x in columns
@@ -260,12 +198,29 @@ class GlueAdapter(SQLAdapter):
 
         return columns
 
+    @available
+    def duplicate_view(self, from_relation: BaseRelation, to_relation: BaseRelation, ):
+        logger.debug("++++++ create_view_as")
+        session, client, cursor = self.get_connection()
+        code = f'''SHOW CREATE TABLE {from_relation.schema}.{from_relation.identifier}'''
+        try:
+            cursor.execute(code)
+            for record in cursor.fetchall():
+                create_view_statement = record[0]
+        except Exception as e:
+            logger.error(e)
+        target_query = create_view_statement.replace(from_relation.schema, to_relation.schema)
+        target_query = target_query.replace(from_relation.identifier, to_relation.identifier)
+        return target_query
+
     @available
     def get_location(self, relation: BaseRelation):
+        logger.debug("++++++ get_location")
         session, client, cursor = self.get_connection()
         return f"LOCATION '{session.credentials.location}/{relation.schema}/{relation.name}/'"
 
     def drop_schema(self, relation: BaseRelation) -> None:
+        logger.debug("++++++ drop_schema")
         session, client, cursor = self.get_connection()
         if self.check_schema_exists(relation.database, relation.schema):
             try:
@@ -280,6 +235,7 @@ class GlueAdapter(SQLAdapter):
             logger.debug("No schema to delete")
 
     def create_schema(self, relation: BaseRelation):
+        logger.debug("++++++ create_schema")
         session, client, cursor = self.get_connection()
         lf = boto3.client("lakeformation", region_name=session.credentials.region)
         sts = boto3.client("sts")
@@ -290,7 +246,6 @@ class GlueAdapter(SQLAdapter):
         else:
             try:
                 # create when database does not exist
-                logger.debug("location = ", session.credentials.location)
                 client.create_database(
                     DatabaseInput={
                         "Name": relation.schema,
@@ -359,29 +314,27 @@ class GlueAdapter(SQLAdapter):
                 logger.error("create_schema exception")
 
     def get_catalog(self, manifest):
+        logger.debug("++++++ get_catalog")
         schema_map = self._get_catalog_schemas(manifest)
-        if len(schema_map) > 1:
-            dbt.exceptions.raise_compiler_error(
-                f'Expected only one database in get_catalog, found '
-                f'{list(schema_map)}'
-            )
 
         with executor(self.config) as tpe:
             futures: List[Future[agate.Table]] = []
             for info, schemas in schema_map.items():
-                for schema in schemas:
-                    futures.append(tpe.submit_connected(
-                        self, schema,
-                        self._get_one_catalog, info, [schema], manifest
-                    ))
+                if len(schemas) == 0:
+                    continue
+                name = list(schemas)[0]
+                fut = tpe.submit_connected(
+                    self, name, self._get_one_catalog, info, [name], manifest
+                )
+                futures.append(fut)
+
             catalogs, exceptions = catch_as_completed(futures)
         return catalogs, exceptions
 
     def _get_one_catalog(
             self, information_schema, schemas, manifest,
     ) -> agate.Table:
-        logger.debug("_get_one_catalog called with args")
-        logger.debug(schemas)
+        logger.debug("++++++ _get_one_catalog")
         if len(schemas) != 1:
             dbt.exceptions.raise_compiler_error(
                 f'Expected only one schema in glue _get_one_catalog, found '
@@ -392,7 +345,6 @@ class GlueAdapter(SQLAdapter):
             schema=list(schemas)[0]
         )
 
-        logger.debug("Schemas : " + schema_base_relation.schema)
         results = self.list_relations_without_caching(schema_base_relation)
         rows = []
 
@@ -404,7 +356,7 @@ class GlueAdapter(SQLAdapter):
 
             for table_row in table_info:
                 rows.append([
-                    information_schema.database,
+                    schema_base_relation.schema,
                     schema_base_relation.schema,
                     name,
                     relation_type,
@@ -430,10 +382,11 @@ class GlueAdapter(SQLAdapter):
         ]
         table = agate.Table(rows, column_names)
 
-        return self._catalog_filter_table(table, manifest)
+        return table
 
     @available
     def create_csv_table(self, model, agate_table):
+        logger.debug("++++++ create_csv_table")
         session, client, cursor = self.get_connection()
         logger.debug(model)
         f = io.StringIO("")
@@ -449,55 +402,27 @@ writer = (
     .option("path", "{session.credentials.location}/{model["schema"]}/{model["name"]}")
 )
 writer.saveAsTable(table_name, mode="append")
-SqlWrapper2.execute("""select * from {model["schema"]}.{model["name"]}""")
+SqlWrapper2.execute("""select * from {model["schema"]}.{model["name"]} limit 1""")
 '''
         try:
             cursor.execute(code)
         except Exception as e:
             logger.error(e)
-            logger.error("create_csv_table exception")
 
 
-    def add_schema_to_cache(self, schema) -> str:
-        if schema is None:
-            name = self.nice_connection_name()
-            dbt.exceptions.raise_compiler_error(
-                'Attempted to cache a null schema for {}'.format(name)
-            )
-        if dbt.flags.USE_CACHE:
-            self.cache.add_schema(None, schema)
-        # so jinja doesn't render things
-        return ''
-
-    @available
-    def describe_table(self, relation):
-        session, client, cursor = self.get_connection()
-        relations = []
-        try:
-            response = client.get_table(
-                DatabaseName=relation.schema,
-                Name=relation.name
-            )
-            relations.append(self.Relation.create(
-                schema=relation.schema,
-                identifier=relation.name,
-                type=self.relation_type_map.get(response.get("Table", {}).get("TableType", "Table"))
-            ))
-            logger.debug(f"""table_name : {relation.name}
-                             table type {self.relation_type_map.get(response.get("Table", {}).get("TableType", "Table"))}
-            """)
-            return relations
-        except Exception:
-            return None
-
     @available
     def get_table_type(self, relation):
+        logger.debug("++++++ get_table_type")
         session, client, cursor = self.get_connection()
         try:
             response = client.get_table(
                 DatabaseName=relation.schema,
                 Name=relation.name
             )
+        except client.exceptions.EntityNotFoundException as e:
+            logger.debug(e)
+            pass
+        try:
             type = self.relation_type_map.get(response.get("Table", {}).get("TableType", "Table"))
             logger.debug("table_name : " + relation.name)
             logger.debug("table type : " + type)
@@ -505,35 +430,8 @@ SqlWrapper2.execute("""select * from {model["schema"]}.{model["name"]}""")
         except Exception as e:
             return None
 
-    def get_rows_different_sql(
-            self,
-            relation_a: BaseRelation,
-            relation_b: BaseRelation,
-            column_names: Optional[List[str]] = None,
-            except_operator: str = 'EXCEPT',
-    ) -> str:
-        """Generate SQL for a query that returns a single row with a two
-        columns: the number of rows that are different between the two
-        relations and the number of mismatched rows.
-        """
-        # This method only really exists for test reasons.
-        names: List[str]
-        if column_names is None:
-            columns = self.get_columns_in_relation(relation_a)
-            names = sorted((self.quote(c.name) for c in columns))
-        else:
-            names = sorted((self.quote(n) for n in column_names))
-        columns_csv = ', '.join(names)
-
-        sql = COLUMNS_EQUAL_SQL.format(
-            columns=columns_csv,
-            relation_a=str(relation_a),
-            relation_b=str(relation_b),
-        )
-
-        return sql
-
     def hudi_write(self, write_mode, session, target_relation, custom_location):
+        logger.debug("++++++ hudi_write")
         if custom_location == "empty":
             return f'''outputDf.write.format('org.apache.hudi').options(**combinedConf).mode('{write_mode}').save("{session.credentials.location}/{target_relation.schema}/{target_relation.name}/")'''
         else:
@@ -542,15 +440,13 @@ SqlWrapper2.execute("""select * from {model["schema"]}.{model["name"]}""")
 
     @available
     def hudi_merge_table(self, target_relation, request, primary_key, partition_key, custom_location):
+        logger.debug("++++++ hudi_merge_table")
         session, client, cursor = self.get_connection()
         isTableExists = False
         if self.check_relation_exists(target_relation):
             isTableExists = True
-            logger.debug(target_relation.schema + '.' + target_relation.name + ' exists.')
         else:
             isTableExists = False
-            logger.debug(
-                 target_relation.schema + '.' + target_relation.name + ' does not exist. Table will be created.')
 
         head_code = f'''
 custom_glue_code_for_dbt_adapter
@@ -609,39 +505,3 @@ SqlWrapper2.execute("""SELECT * FROM {target_relation.schema}.{target_relation.n
             cursor.execute(code)
         except Exception as e:
             logger.error(e)
-            logger.error("hudi_merge_table exception")
-
-
-# spark does something interesting with joins when both tables have the same
-# static values for the join condition and complains that the join condition is
-# "trivial". Which is true, though it seems like an unreasonable cause for
-# failure! It also doesn't like the `from foo, bar` syntax as opposed to
-# `from foo cross join bar`.
-COLUMNS_EQUAL_SQL = '''
-with diff_count as (
-    SELECT
-        1 as id,
-        COUNT(*) as num_missing FROM (
-            (SELECT {columns} FROM {relation_a} EXCEPT
-             SELECT {columns} FROM {relation_b})
-             UNION ALL
-            (SELECT {columns} FROM {relation_b} EXCEPT
-             SELECT {columns} FROM {relation_a})
-        ) as a
-), table_a as (
-    SELECT COUNT(*) as num_rows FROM {relation_a}
-), table_b as (
-    SELECT COUNT(*) as num_rows FROM {relation_b}
-), row_count_diff as (
-    select
-        1 as id,
-        table_a.num_rows - table_b.num_rows as difference
-    from table_a
-    cross join table_b
-)
-select
-    row_count_diff.difference as row_count_difference,
-    diff_count.num_missing as num_mismatched
-from row_count_diff
-cross join diff_count
-'''.strip()
