@@ -4,8 +4,8 @@
   {%- set raw_file_format = config.get('file_format', default='parquet') -%}
   {%- set raw_strategy = config.get('incremental_strategy', default='insert_overwrite') -%}
   
-  {%- set file_format = dbt_glue_validate_get_file_format(raw_file_format) -%}
-  {%- set strategy = dbt_glue_validate_get_incremental_strategy(raw_strategy, file_format) -%}
+  {%- set file_format = dbt_spark_validate_get_file_format(raw_file_format) -%}
+  {%- set strategy = dbt_spark_validate_get_incremental_strategy(raw_strategy, file_format) -%}
 
   {%- set unique_key = config.get('unique_key', none) -%}
   {%- set partition_by = config.get('partition_by', none) -%}
@@ -14,10 +14,9 @@
   {%- set full_refresh_mode = (flags.FULL_REFRESH == True) -%}
 
   {% set target_relation = this %}
-  {% set existing_relation_type = adapter.get_table_type(this)  %}
-  {% set tmp_relation = glue__make_temp_relation(this) %}
-
-  {{ run_hooks(pre_hooks) }}
+  {% set existing_relation_type = adapter.get_table_type(target_relation)  %}
+  {% set tmp_relation = make_temp_relation(target_relation, '_tmp') %}
+  {% set is_incremental = 'False' %}
 
   {% if raw_strategy == 'merge' and file_format == 'hudi' %}
         {{ adapter.hudi_merge_table(target_relation, sql, unique_key, partition_by, custom_location) }}
@@ -28,14 +27,14 @@
           set glue.sql.sources.partitionOverwriteMode = DYNAMIC
         {% endcall %}
       {% endif %}
-
       {% if existing_relation_type is none %}
         {% set build_sql = create_table_as(False, target_relation, sql) %}
       {% elif existing_relation_type == 'view' or full_refresh_mode %}
-        {% do adapter.drop_view(target_relation) %}
+        {{ drop_relation(target_relation) }}
         {% set build_sql = create_table_as(False, target_relation, sql) %}
       {% else %}
-        {{ adapter.create_view_as(tmp_relation, sql) }}
+        {{ glue__create_view(tmp_relation, sql) }}
+        {% set is_incremental = 'True' %}
         {% set build_sql = dbt_glue_get_incremental_sql(strategy, tmp_relation, target_relation, unique_key) %}
       {% endif %}
   {% endif %}
@@ -44,8 +43,9 @@
      {{ build_sql }}
   {%- endcall -%}
 
-  {{ adapter.drop_view(tmp_relation) }}
-  {{ run_hooks(post_hooks) }}
+  {% if is_incremental == 'True' %}
+    {{ glue__drop_view(tmp_relation) }}
+  {% endif %}
 
   {{ return({'relations': [target_relation]}) }}
 
